package com.hisschemoller.model 
{
	import com.hisschemoller.model.audio.IAudioProcessor;
	import com.hisschemoller.model.vo.AudioEventVO;

	import org.puremvc.as3.interfaces.IProxy;
	import org.puremvc.as3.patterns.proxy.Proxy;	
	/**
	 * @author Wouter Hisschemöller
	 * (c) Jul 17, 2009
	 */
	public class AudioEngineProxy extends Proxy implements IProxy 
	{
		public static const NAME : String = "AudioEngineProxy";
		public static const NOTE_ADDED : String = "noteAdded";
		public static const NOTE_REMOVED : String = "noteRemoved";
		private var _generators : Vector.<AudioEventVO> = new Vector.<AudioEventVO>();

		public function AudioEngineProxy()
		{
			super(NAME);
		}

		/**
		 * 
		 */
		public function process(buffer : Vector.<Vector.<Number>>) : void
		{
			var n : int = _generators.length;
			
			while(--n > -1)
			{
				// IAudioProcessor returns true if the sound is finished
				if(_generators[n].audioGenerator.process(buffer))
				{
					sendNotification(NOTE_REMOVED, _generators[n]);
					_generators.splice(n, 1);
				}
			}
		}

		/**
		 * 
		 */
		public function addGenerator(audioEventVO : AudioEventVO) : void
		{
			// first remove possible identical events that are still playing
			removeGenerator(audioEventVO);
			
			// Do not add silent audio events (with velocity zero, volume zero).
			// Besides, velocity zero means 'note off' or end of the event.
			if(audioEventVO.velocity == 0) return;
			
			// add the event to the list of all events.
			_generators.splice(0, 0, audioEventVO);
			
			sendNotification(NOTE_ADDED, audioEventVO);
		}

		/**
		 * 
		 */
		public function removeGenerator(inAudioEventVO : AudioEventVO) : void
		{
			var n : int = _generators.length;
			
			while(--n > -1)
			{
				// audio events are identified by their channel and pitch
				if((_generators[n].channel == inAudioEventVO.channel) && (_generators[n].pitch == inAudioEventVO.pitch))
				{
					sendNotification(NOTE_REMOVED, _generators[n]);
					_generators.splice(n, 1);
				}
			}
		}

		/**
		 * 
		 */
		public function removeAllGenerators() : void
		{
			var n : int = _generators.length;
			
			while(--n > -1)
			{
				sendNotification(NOTE_REMOVED, _generators[n]);
			}
			
			_generators = new Vector.<AudioEventVO>();
		}
	}
}
