package com.hisschemoller.view 
{
	import com.hisschemoller.AppFacade;
	import com.hisschemoller.model.SampleBankProxy;
	import com.hisschemoller.model.SampleEditorProxy;
	import com.hisschemoller.model.WaveDisplayProxy;
	import com.hisschemoller.model.vo.SamplerPresetVO;
	import com.hisschemoller.view.components.SampleEditor;
	
	import org.puremvc.as3.interfaces.IMediator;
	import org.puremvc.as3.interfaces.INotification;
	import org.puremvc.as3.patterns.mediator.Mediator;
	
	import flash.events.Event;	
	/**
	 * (c) Copyright LBi Lost Boys 2009
	 * @author wouter.hisschemoller
	 */
	public class SampleEditorMediator extends Mediator implements IMediator 
	{
		public static const NAME : String = "SampleEditorMediator";
		private var _sampleEditorProxy : SampleEditorProxy;
		private var _displayProxy : WaveDisplayProxy;

		public function SampleEditorMediator(viewComponent : Object)
		{
			super(NAME, viewComponent);
		}

		/**
		 * 
		 */
		override public function onRegister() : void
		{
			_sampleEditorProxy = facade.retrieveProxy(SampleEditorProxy.NAME) as SampleEditorProxy;
			_displayProxy = facade.retrieveProxy(WaveDisplayProxy.NAME) as WaveDisplayProxy;
			
			sampleEditor.addEventListener(SampleEditor.START_PLAY_LOCATOR_CHANGE, handleLocatorChange);
			sampleEditor.addEventListener(SampleEditor.END_PLAY_LOCATOR_CHANGE, handleLocatorChange);
			sampleEditor.addEventListener(SampleEditor.START_LOOP_LOCATOR_CHANGE, handleLocatorChange);
			sampleEditor.addEventListener(SampleEditor.END_LOOP_LOCATOR_CHANGE, handleLocatorChange);
			sampleEditor.addEventListener(SampleEditor.REVERSE_CHANGE, reverseChangeHandler);
			sampleEditor.addEventListener(SampleEditor.TRANSPOSE_CHANGE, transposeChangeHandler);
		}

		/**
		 * 
		 */
		override public function listNotificationInterests() : Array
		{
			return [ SampleBankProxy.PRESET_CHANGED,
					WaveDisplayProxy.GRAPHIC_UPDATED,
					SampleEditorProxy.SAMPLE_POSITIONS_CHANGED,
					SampleEditorProxy.SAMPLE_TRANSPOSE_CHANGED  ];
		}

		/**
		 * 
		 */
		override public function handleNotification(note : INotification) : void
		{
			var preset : SamplerPresetVO = (facade.retrieveProxy(SampleBankProxy.NAME) as SampleBankProxy).selectedPreset;
			
			var startValue : Number = (preset.start - _displayProxy.firstSampleInView) / _displayProxy.numSamplesInView;
			var endValue : Number = (preset.end - _displayProxy.firstSampleInView) / _displayProxy.numSamplesInView;
			
			switch(note.getName())
			{
				case SampleBankProxy.PRESET_CHANGED:
					sampleEditor.updateLocators(startValue, endValue, preset.start, preset.end);
					sampleEditor.transpose = preset.transpose;
					sampleEditor.reverse = preset.reverse;
					break;
				case WaveDisplayProxy.GRAPHIC_UPDATED:
					sampleEditor.updateLocators(startValue, endValue, preset.start, preset.end);
					break;
				case SampleEditorProxy.SAMPLE_POSITIONS_CHANGED:
					sampleEditor.updateLocators(startValue, endValue, preset.start, preset.end);
					break;
				case SampleEditorProxy.SAMPLE_TRANSPOSE_CHANGED:
					sampleEditor.transpose = preset.transpose;
					break;
			}
		}

		/**
		 * Called when a locator is dragged
		 * @param event : event whose type indicates which locator was dragged.
		 */
		private function handleLocatorChange(event : Event) : void
		{
			var sampleIndex : int = _displayProxy.firstSampleInView + Math.round(_displayProxy.numSamplesInView * sampleEditor.getLocatorValue(event.type));
			
			switch(event.type)
			{
				case SampleEditor.START_PLAY_LOCATOR_CHANGE:
					_sampleEditorProxy.presetStart = sampleIndex;
					break;
				case SampleEditor.END_PLAY_LOCATOR_CHANGE:
					_sampleEditorProxy.presetEnd = sampleIndex;
					break;
				case SampleEditor.START_LOOP_LOCATOR_CHANGE:
					//
					break;
				case SampleEditor.END_LOOP_LOCATOR_CHANGE:
					//
					break;
			}
		}

		/**
		 * Called when the reverse button is clicked.
		 */
		private function reverseChangeHandler(event : Event) : void
		{
			facade.sendNotification(AppFacade.REVERSE_SAMPLE, sampleEditor.reverse);
		}

		/**
		 * Called when the reverse button is clicked.
		 */
		private function transposeChangeHandler(event : Event) : void
		{
			_sampleEditorProxy.presetTranspose = sampleEditor.transpose;
		}

		/**
		 * 
		 */
		private function get sampleEditor() : SampleEditor
		{
			return viewComponent as SampleEditor;
		}
	}
}
