package com.hisschemoller.view 
{
	import com.hisschemoller.model.SampleBankProxy;
	import com.hisschemoller.model.WaveDisplayProxy;
	import com.hisschemoller.view.components.WaveDisplay;
	
	import org.puremvc.as3.interfaces.IMediator;
	import org.puremvc.as3.interfaces.INotification;
	import org.puremvc.as3.patterns.mediator.Mediator;
	
	import flash.display.BitmapData;
	import flash.events.Event;
	import flash.events.MouseEvent;	
	/**
	 * (c) Copyright LBi Lost Boys 2009
	 * @author wouter.hisschemoller
	 */
	public class WaveDisplayMediator extends Mediator implements IMediator 
	{
		public static const NAME : String = "WaveDisplayMediator";
		private var _displayProxy : WaveDisplayProxy;

		public function WaveDisplayMediator(viewComponent : Object)
		{
			super(NAME, viewComponent);
		}

		/**
		 * 
		 */
		override public function onRegister() : void
		{
			_displayProxy = facade.retrieveProxy(WaveDisplayProxy.NAME) as WaveDisplayProxy;
			
			waveDisplay.addEventListener(WaveDisplay.HORIZONTAL_ZOOM_CHANGE, handleHZoomSliderChange);
			waveDisplay.addEventListener(WaveDisplay.VERTICAL_ZOOM_CHANGE, handleVZoomSliderChange);
			waveDisplay.addEventListener(WaveDisplay.POSITION_CHANGE, handlePositionSliderChange);
			waveDisplay.addEventListener(MouseEvent.MOUSE_DOWN, handleGraphicMouseDown);
			waveDisplay.addEventListener(WaveDisplay.MOUSE_DRAG, handleGraphicMouseDrag);
			waveDisplay.addEventListener(MouseEvent.DOUBLE_CLICK, graphicDoubleClickHandler);
		}

		/**
		 * 
		 */
		override public function listNotificationInterests() : Array
		{
			return [SampleBankProxy.PRESET_CHANGED,
					WaveDisplayProxy.GRAPHIC_UPDATED,
					WaveDisplayProxy.ZOOM_UPDATED ];
		}

		/**
		 * 
		 */
		override public function handleNotification(note : INotification) : void
		{
			switch(note.getName())
			{
				case SampleBankProxy.PRESET_CHANGED:
					waveDisplay.updateHZoomSlider(_displayProxy.horizontalZoomLevel);
					waveDisplay.updateVZoomSlider(0);
					break;
				case WaveDisplayProxy.GRAPHIC_UPDATED:
					waveDisplay.updateGraphic(note.getBody() as BitmapData);
					waveDisplay.updatePosition(_displayProxy.numSamplesInView, _displayProxy.numSamplesTotal, _displayProxy.firstSampleInView, _displayProxy.currentSamplesPerPixel);
					break;
				case WaveDisplayProxy.ZOOM_UPDATED:
					waveDisplay.updateHZoomSlider(_displayProxy.horizontalZoomLevel);
					break;
			}
		}

		/**
		 * 
		 */
		private function handleHZoomSliderChange(event : Event) : void
		{
			_displayProxy.horizontalZoomLevel = waveDisplay.hZoomSliderValue;
		}

		/**
		 * 
		 */
		private function handleVZoomSliderChange(event : Event) : void
		{
			_displayProxy.verticalZoomLevel = waveDisplay.vZoomSliderValue;
		}

		/**
		 * 
		 */
		private function handlePositionSliderChange(event : Event) : void
		{
			_displayProxy.firstSampleInView = waveDisplay.positionScrollBarPosition;
		}

		/**
		 * Mouse is pressed on the wave display area.
		 */
		private function handleGraphicMouseDown(event : MouseEvent) : void
		{
			_displayProxy.initializeDrag(waveDisplay.graphicMouseX, waveDisplay.graphicMouseY);
		}

		/**
		 * Mouse is dragged on the wave display area.
		 */
		private function handleGraphicMouseDrag(event : Event) : void
		{
			_displayProxy.updateDrag(waveDisplay.graphicMouseX, waveDisplay.graphicMouseY);
		}

		/**
		 * Mouse is dragged on the wave display area.
		 */
		private function graphicDoubleClickHandler(event : MouseEvent) : void
		{
			_displayProxy.horizontalZoomLevel = 1;
		}

		/**
		 * 
		 */
		private function get waveDisplay() : WaveDisplay
		{
			return viewComponent as WaveDisplay;
		}
	}
}
