package com.hisschemoller.view.components.base
{
	import flash.display.DisplayObject;
	import flash.display.DisplayObjectContainer;
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.EventDispatcher;	
	/**
	 * (c) Copyright LBi Lost Boys 2009
	 * @author wouter.hisschemoller
	 */
	public class Component extends EventDispatcher 
	{
		public static const DRAW : String = "draw";
		protected var _asset : MovieClip;
		protected var _width : Number = 0;
		protected var _height : Number = 0;

		public function Component(inContainer : DisplayObjectContainer = null, inAsset : MovieClip = null)
		{
			_asset = inAsset;
			
			if(inContainer && _asset && !inContainer.contains(_asset))
			{
				inContainer.addChild(_asset);
			}
			
			init();
		}

		/**
		 * Moves the component to the specified position.
		 * @param xpos the x position to move the component
		 * @param ypos the y position to move the component
		 */
		public function move(inXPos : Number, inYPos : Number) : void
		{
			_asset.x = Math.round(inXPos);
			_asset.y = Math.round(inYPos);
		}

		/**
		 * Sets the size of the component.
		 * @param inWidth The width of the component.
		 * @param inHeight The height of the component.
		 */
		public function setSize(inWidth : Number, inHeight : Number) : void
		{
			_width = inWidth;
			_height = inHeight;
			
			invalidate();
		}

		/**
		 * Abstract draw function.
		 */
		public function draw() : void
		{
			dispatchEvent(new Event(Component.DRAW));
		}

		/**
		 * Initilizes the component.
		 */
		protected function init() : void
		{
			addChildren();
			invalidate();
		}

		/**
		 * Overriden in subclasses to create child display objects.
		 */
		protected function addChildren() : void
		{
		}

		/**
		 * Marks the component to be redrawn on the next frame.
		 */
		protected function invalidate() : void
		{
			_asset.addEventListener(Event.ENTER_FRAME, handleInvalidate);
		}

		/**
		 * Called one frame after invalidate is called.
		 */
		private function handleInvalidate(event : Event) : void
		{
			_asset.removeEventListener(Event.ENTER_FRAME, handleInvalidate);
			draw();
		}
	}
}
